"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = void 0;
const prerender_manifest_json_1 = __importDefault(require("./prerender-manifest.json"));
const manifest_json_1 = __importDefault(require("./manifest.json"));
const routes_manifest_json_1 = require("./routes-manifest.json");
const thrive_next_aws_cloudfront_1 = __importDefault(require("thrive-next-aws-cloudfront"));
const s3_1 = __importDefault(require("aws-sdk/clients/s3"));
const addS3HostHeader = (req, s3DomainName) => {
    req.headers["host"] = [{ key: "host", value: s3DomainName }];
};
const isDataRequest = (uri) => uri.startsWith("/_next/data");
const normaliseUri = (uri) => {
    if (routes_manifest_json_1.basePath)
        uri = uri.slice(routes_manifest_json_1.basePath.length);
    return uri === "" ? "/index" : uri;
};
const normaliseS3OriginDomain = (s3Origin) => {
    if (s3Origin.region === "us-east-1") {
        return s3Origin.domainName;
    }
    if (!s3Origin.domainName.includes(s3Origin.region)) {
        const regionalEndpoint = s3Origin.domainName.replace("s3.amazonaws.com", `s3.${s3Origin.region}.amazonaws.com`);
        return regionalEndpoint;
    }
    return s3Origin.domainName;
};
const router = (manifest) => {
    const { pages: { ssr, html } } = manifest;
    const allDynamicRoutes = Object.assign(Object.assign({}, ssr.dynamic), html.dynamic);
    return (uri) => {
        let normalisedUri = uri;
        if (isDataRequest(uri)) {
            normalisedUri = uri
                .replace(`/_next/data/${manifest.buildId}`, "")
                .replace(".json", "");
        }
        if (ssr.nonDynamic[normalisedUri]) {
            return ssr.nonDynamic[normalisedUri];
        }
        for (const route in allDynamicRoutes) {
            const { file, regex } = allDynamicRoutes[route];
            const re = new RegExp(regex, "i");
            const pathMatchesRoute = re.test(normalisedUri);
            if (pathMatchesRoute) {
                return file;
            }
        }
        if (html.nonDynamic["/404"] !== undefined) {
            return "pages/404.html";
        }
        return "pages/_error.js";
    };
};
exports.handler = (event) => __awaiter(void 0, void 0, void 0, function* () {
    const manifest = manifest_json_1.default;
    const prerenderManifest = prerender_manifest_json_1.default;
    if (isOriginResponse(event)) {
        return yield handleOriginResponse({ event, manifest, prerenderManifest });
    }
    else {
        return yield handleOriginRequest({ event, manifest, prerenderManifest });
    }
});
const handleOriginRequest = ({ event, manifest, prerenderManifest }) => __awaiter(void 0, void 0, void 0, function* () {
    const request = event.Records[0].cf.request;
    const uri = normaliseUri(request.uri);
    const { pages, publicFiles } = manifest;
    const isStaticPage = pages.html.nonDynamic[uri];
    const isPublicFile = publicFiles[uri];
    const isPrerenderedPage = prerenderManifest.routes[uri];
    const origin = request.origin;
    const s3Origin = origin.s3;
    const isHTMLPage = isStaticPage || isPrerenderedPage;
    const normalisedS3DomainName = normaliseS3OriginDomain(s3Origin);
    const hasFallback = hasFallbackForUri(uri, prerenderManifest);
    s3Origin.domainName = normalisedS3DomainName;
    if (isHTMLPage || isPublicFile || hasFallback || isDataRequest(uri)) {
        if (isHTMLPage || hasFallback) {
            s3Origin.path = `${routes_manifest_json_1.basePath}/static-pages`;
            let pageName = uri === "/" ? "/index" : uri;
            request.uri = `${pageName}.html`;
        }
        if (isPublicFile) {
            s3Origin.path = `${routes_manifest_json_1.basePath}/public`;
            if (routes_manifest_json_1.basePath) {
                request.uri = request.uri.replace(routes_manifest_json_1.basePath, "");
            }
        }
        addS3HostHeader(request, normalisedS3DomainName);
        return request;
    }
    const pagePath = router(manifest)(uri);
    if (pagePath.endsWith(".html")) {
        s3Origin.path = `${routes_manifest_json_1.basePath}/static-pages`;
        request.uri = pagePath.replace("pages", "");
        addS3HostHeader(request, normalisedS3DomainName);
        return request;
    }
    const page = require(`./${pagePath}`);
    const { req, res, responsePromise } = thrive_next_aws_cloudfront_1.default(event.Records[0].cf);
    page.render(req, res);
    return responsePromise;
});
const handleOriginResponse = ({ event, manifest, prerenderManifest }) => __awaiter(void 0, void 0, void 0, function* () {
    var _a, _b;
    const response = event.Records[0].cf.response;
    const request = event.Records[0].cf.request;
    const uri = normaliseUri(request.uri);
    const { status } = response;
    if (status !== "403")
        return response;
    const { domainName, region } = request.origin.s3;
    const bucketName = domainName.replace(`.s3.${region}.amazonaws.com`, "");
    const s3 = new s3_1.default({ region: (_b = (_a = request.origin) === null || _a === void 0 ? void 0 : _a.s3) === null || _b === void 0 ? void 0 : _b.region });
    if (isDataRequest(uri)) {
        const pagePath = router(manifest)(uri);
        const page = require(`./${pagePath}`);
        const { req, res, responsePromise } = thrive_next_aws_cloudfront_1.default(event.Records[0].cf);
        const isSSG = !!page.getStaticProps;
        const { renderOpts, html } = yield page.renderReqToHTML(req, res, "passthrough");
        if (isSSG) {
            const s3JsonParams = {
                Bucket: bucketName,
                Key: uri.replace(/^\//, ""),
                Body: JSON.stringify(renderOpts.pageData),
                ContentType: "application/json"
            };
            const s3HtmlParams = {
                Bucket: bucketName,
                Key: `static-pages/${request.uri
                    .replace(`/_next/data/${manifest.buildId}/`, "")
                    .replace(".json", ".html")}`,
                Body: html,
                ContentType: "text/html",
                CacheControl: "public, max-age=0, s-maxage=2678400, must-revalidate"
            };
            yield Promise.all([
                s3.putObject(s3JsonParams).promise(),
                s3.putObject(s3HtmlParams).promise()
            ]);
        }
        res.writeHead(200, response.headers);
        res.setHeader("Content-Type", "application/json");
        res.end(JSON.stringify(renderOpts.pageData));
        return responsePromise;
    }
    else {
        const hasFallback = hasFallbackForUri(uri, prerenderManifest);
        if (!hasFallback)
            return response;
        const s3Params = {
            Bucket: bucketName,
            Key: `static-pages${hasFallback.fallback}`
        };
        const { Body } = yield s3.getObject(s3Params).promise();
        return {
            status: "200",
            statusDescription: "OK",
            headers: Object.assign(Object.assign({}, response.headers), { "content-type": [
                    {
                        key: "Content-Type",
                        value: "text/html"
                    }
                ] }),
            body: Body === null || Body === void 0 ? void 0 : Body.toString("utf-8")
        };
    }
});
const isOriginResponse = (event) => {
    return event.Records[0].cf.config.eventType === "origin-response";
};
const hasFallbackForUri = (uri, prerenderManifest) => {
    return Object.values(prerenderManifest.dynamicRoutes).find((routeConfig) => {
        const re = new RegExp(routeConfig.routeRegex);
        return re.test(uri);
    });
};
